MODULE declarations
IMPLICIT NONE

DOUBLE PRECISION, PUBLIC, ALLOCATABLE,  DIMENSION (:)     :: x, x_face, y, y_face, tid_vek, mass_tot, mass_rate_tot
DOUBLE PRECISION, PUBLIC, ALLOCATABLE,  DIMENSION (:)	  :: h_a_vek, q_tot, mass_tot_avd
DOUBLE PRECISION, PUBLIC, ALLOCATABLE,  DIMENSION (:,:,:) :: U, F, G, Flf, Glf
INTEGER, PUBLIC				                              :: npi,npj,npk,iter_out,iter_pub,cell_rel
DOUBLE  PRECISION, PUBLIC, ALLOCATABLE, DIMENSION(:,:)    :: s_vap,s_vap_e, q_u, m_vap, tid_evap, h_last
DOUBLE PRECISION, PUBLIC							   	  :: tid, alpha
REAL, PUBLIC										      :: grav,t_end,dt, masse_tot
DOUBLE PRECISION, PARAMETER 						  	  :: eps=1e-07, T_dam=111 , T_u=290, pi=3.1415929
REAL, PARAMETER											  :: dia_o=0.5,dia_t=5.,Cd=0.5,Q=2. !Q er volumstrmmen ut av tanken 
REAL, PARAMETER											  :: H0=3., tid_dur=10.
CHARACTER(len=1), PUBLIC 								  :: determ1, determ2



!Typevariable med egenskaper:
! typiske varmeovergangstall
TYPE, PUBLIC :: varme_par
 REAL :: h_kim,h_over,h_film,h_fgm,rho_m
 REAL :: rho_g, Cp_g, gamma, T_jord, T_a, Pr_a, ny_a, k_a
END TYPE

TYPE(varme_par), PUBLIC :: varme_o


END MODULE declarations

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

MODULE sub
USE declarations
CONTAINS

SUBROUTINE read_prop()
USE declarations
IMPLICIT NONE
!-------------------------------------------------------
!Hensikt: Leser inn konstanter fra tekstfilen varme.txt
!-------------------------------------------------------
OPEN(10,file='varme.txt',status='unknown')
READ(10,'(F9.5)') varme_o
CLOSE(10)


END SUBROUTINE read_prop

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE grid(dx,dy)
IMPLICIT NONE

INTEGER :: i,j
DOUBLE PRECISION 			  :: xl, yl 
DOUBLE PRECISION, INTENT (out):: dx, dy
!--------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Ordner netteverk og skriver dem ut til x.dat og
!y.dat
!--------------------------------------------------------

!Tall p antall noder i nettverk:
WRITE(*,*) 'Antall kontrollvolumer i x-retning:'
READ(*,*) npi
npi=npi+2
WRITE(*,*) 'Antall kontrollvolumer i y-retning:'
READ(*,*) npj
npj=npj+2
!Skal vsken spredes?
WRITE(*,*)
WRITE(*,*) 'Med damspredning? (Y/N):'
READ(*,*) determ1
WRITE(*,*)
!Avdamping p?
WRITE(*,*) 'Med avdamping? (Y/N):'
READ(*,*) determ2
WRITE(*,*)


IF (determ1=='n' .and. determ2=='n') THEN
  
WRITE(*,*)'Ingen beregning utfrt'
STOP

END IF

ALLOCATE(x(npi),x_face(npi),y(npj),y_face(npj))



!Beregningsomrde
xl=100
yl=100

!Lengder i kontrollvolumene:
dx=xl/real(npi-2)
dy=yl/real(npj-2)

!Grid:
!x-retning
x(1)=0.
x(2)=dx*0.5
DO i=3,npi-1
  x(i)=x(i-1)+dx
END DO
x(npi)=x(npi-1)+0.5*dx

!y-retning
y(1)=0.
y(2)=dy*0.5
DO j=3,npj-1
  y(j)=y(j-1)+dy
END DO
y(npj)=y(npj-1)+0.5*dy

!Grenseflater:                     
!x-retning:
x_face(1)=0.
x_face(2)=0.
DO i=3,npi
  x_face(i)=x_face(i-1)+dx
END DO

!y-retning
y_face(1)=0.
y_face(2)=0.
DO j=3,npj
  y_face(j)=y_face(j-1)+dx
END DO


CALL print_grid()


END SUBROUTINE grid

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE outblock_and_source(dx,dy)
USE declarations
IMPLICIT NONE	

DOUBLE PRECISION										:: lengde,dx,deter,dy
INTEGER													:: i,j
REAL 													:: h_tank, u_b
!-------------------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Utblokkerer vegger og setter indre randbetingelser for kilder
!-------------------------------------------------------------------------------

h_tank=(SQRT(H0)-Cd*((dia_o/dia_t)**2)*SQRT(grav/2)*tid)**2

!utblokkerer tank
DO i=1,npi
	DO j=1,npj
    deter=SQRT(((x(i)-(dia_t/2))**2+(y(j)-y_face(npj/2))**2))
	IF (deter<=dia_t/2) THEN
  	
		Flf(i,j,:)=0.
		Glf(i,j,:)=0.
        
    END IF
	END DO
END DO        
   

cell_rel=ceiling(dia_t/dx)+1
	
    
     IF (tid<=tid_dur .and. tid>0.3) THEN
       
      U(cell_rel,npj/2-1:npj/2,1)=dia_o 
      U(cell_rel,npj/2-1:npj/2,2)=(Q*tid_dur-2*dy*dx*dia_o)/(2*dy*tid_dur)  
      U(cell_rel,npj/2-1:npj/2,3)=0.
      
     ELSE  
        
      U(cell_rel,npj/2-1:npj/2,2)=0.
      
  	 END IF
     

END SUBROUTINE outblock_and_source

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


SUBROUTINE init(dx,dy)
USE declarations
IMPLICIT NONE

INTEGER 						:: iter
DOUBLE PRECISION, INTENT (in)	:: dx,dy
DOUBLE PRECISION				:: Cmax, v_ut
!-----------------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Initialiserer domenet samt allokerer arrays
! NB! Her kan man sette t_end (hvor lenge simuleringen varer)
!-----------------------------------------------------------------------------

alpha=varme_o%gamma/(varme_o%rho_g*varme_o%Cp_g) !termisk diffusivitet underlag


ALLOCATE(U(npi,npj,3))  !U inneholder 3 variable i hver celle
ALLOCATE(s_vap(npi,npj), q_u(npi,npj), m_vap(npi,npj), tid_evap(npi,npj), h_last(npi,npj)) !reg.hast, varmefluks, massefluks(massetransport pga avdampning)
ALLOCATE(s_vap_e(npi,npj))

!Initierer 
s_vap=0.
s_vap_e=0.
q_u=0.
m_vap=0.
tid_evap=0.1

! Setter parametre
grav=9.81
t_end=200.


!Ber om input Courant-tall:
WRITE(*,*)
WRITE(*,*)'Skriv inn courant-tall (anb. Cmax=1.0):'
READ(*,*)Cmax
WRITE(*,*)

!hastighet ut av hull v/ tid=0s
v_ut=(Q*tid_dur-2*dy*dx*dia_o)/(2*dy*tid_dur*dia_o)		 

!Allokerer fluksvektorer
ALLOCATE(F(npi,npj,3),G(npi,npj,3),Flf(npi,npj,3),Glf(npi,npj,3)) 

!----Initialiserer variable

 	    U(:,:,1)=eps
        U(:,:,2)=0.
		U(:,:,3)=0.

		
!regner ut dt ut fra stab.krit:
dt=Cmax*min(dx,dy)/(v_ut+2*SQRT(grav*dia_o))

WRITE(*,*)
WRITE(*,*)'Tidsskritt:',dt
WRITE(*,*)

iter=CEILING(t_end/dt) !ant. iterasjoner

!output hver:
iter_out=CEILING(5./dt)

!Allokerer tidsvektorer
ALLOCATE(mass_tot(iter+1), tid_vek(iter+1), mass_rate_tot(iter+1), h_a_vek(iter+1))
ALLOCATE(q_tot(iter+1),mass_tot_avd(iter+1))

!Initierer
mass_tot=0.
tid_vek=0.
mass_rate_tot=0.
h_a_vek=0.
q_tot=0.
mass_tot_avd=0.

h_last=U(:,:,1) !vet hva hyden var p forrige tidsskritt
WRITE(*,*)
WRITE(*,*)'Initialbetingelser satt.'
WRITE(*,*)
RETURN
END SUBROUTINE init

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE printout(iternr)
USE declarations
IMPLICIT NONE

INTEGER 						:: i,j, iternr
CHARACTER(len=6)			    :: tidsstring
CHARACTER(len=50)			    :: filnavn1, filnavn2, filnavn3, filnavn4
CHARACTER(len=50)				:: filnavn6, filnavn7, filnavn8, filnavn9
!---------------------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Printer ut variable ved iterasjon "iterout"
!---------------------------------------------------------------------------------

IF ((MOD(iternr,iter_out)==0) .or. tid==0.0) THEN
  	  WRITE(*,*)
      WRITE(*,*)'          Tid(s)  ','                 Iternr.','   Hoyde i sjekknode(m)'
      WRITE(*,*)'      -------------------------------------------------------------------'
      WRITE(*,*) tid,iternr,U(npi/2,npj/2,1) 
      WRITE(*,*)
      WRITE(tidsstring,'(f8.4)') tid
      tidsstring=TRIM(tidsstring)
      filnavn1 = 'res_hoyde'//tidsstring//'.dat'
      filnavn2 = 'res_u'//tidsstring//'.dat'
      filnavn3 = 'res_v'//tidsstring//'.dat'
      filnavn4 = 'res_abshast'//tidsstring//'.dat'
      
      filnavn6 = 'res_q_u'//tidsstring//'.dat'
      filnavn7 = 'res_s_vap'//tidsstring//'.dat'
      filnavn8 = 'res_m_vap'//tidsstring//'.dat'
	  filnavn9 = 'res_s_vap_e'//tidsstring//'.dat'      

      filnavn1=TRIM(filnavn1)
      filnavn2=TRIM(filnavn2)
      filnavn3=TRIM(filnavn3)
	  filnavn4=TRIM(filnavn4)
	  
      filnavn6=TRIM(filnavn6)
      filnavn7=TRIM(filnavn7)
      filnavn8=TRIM(filnavn8)
      filnavn9=TRIM(filnavn9)	
      
OPEN(10,file=filnavn1,status='unknown')
OPEN(11,file=filnavn2,status='unknown')
OPEN(12,file=filnavn3,status='unknown')
OPEN(13,file=filnavn4,status='unknown')
OPEN(15,file=filnavn6,status='unknown')
OPEN(16,file=filnavn7,status='unknown')
OPEN(17,file=filnavn8,status='unknown')
OPEN(18,file=filnavn9,status='unknown')
     
DO i=1,npi
   WRITE(10,102)(U(i,j,1),j=1,npj)
END DO

DO i=1,npi
   WRITE(11,102)(U(i,j,2)/U(i,j,1),j=1,npj)
END DO


DO i=1,npi
   WRITE(12,102)(U(i,j,3)/U(i,j,1),j=1,npj)
END DO

DO i=1,npi
   WRITE(13,102)(SQRT((U(i,j,3)/U(i,j,1))**2+(U(i,j,2)/U(i,j,1))**2),j=1,npj)
END DO  


DO i=1,npi
   WRITE(15,102)(q_u(i,j),j=1,npj)
END DO
 
DO i=1,npi
   WRITE(16,102)(s_vap(i,j),j=1,npj)
END DO

DO i=1,npi
   WRITE(17,102)(m_vap(i,j),j=1,npj)
END DO

DO i=1,npi
   WRITE(18,102)(s_vap_e(i,j),j=1,npj)
END DO

CLOSE(10)
CLOSE(11)
CLOSE(12)
CLOSE(13)
CLOSE(15)
CLOSE(16)
CLOSE(17)
CLOSE(18)

END IF 

102  FORMAT(' ',1000(E15.6E3))

RETURN
END SUBROUTINE printout

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE print_grid()
USE declarations
IMPLICIT NONE

INTEGER				:: i
!--------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Skriver ut nettverket
!--------------------------------------------------------

OPEN(13,file='x.dat',status='unknown')
OPEN(14,file='y.dat',status='unknown')

DO i=1,npi
   WRITE(13,100) x(i)
END DO

DO i=1,npj
   WRITE(14,100) y(i)
END DO  

CLOSE(13)
CLOSE(14)


100  FORMAT(' ',4E15.6E3)

WRITE(*,*)
WRITE(*,*) 'Nettverk skrevet til fil.'
WRITE(*,*)

END SUBROUTINE print_grid

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE local_lf(Un,dx,dy)
USE declarations
IMPLICIT NONE

DOUBLE PRECISION, INTENT(in), DIMENSION(npi,npj,3)		:: Un
DOUBLE PRECISION, DIMENSION(npi,npj)					:: ax,ay
DOUBLE PRECISION, INTENT(in)							:: dx,dy
!-----------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Regner ut fluksfunksjonene p grensene (Rusanov-fluks)
!-----------------------------------------------------------------

CALL make_diffterm(Un,ax,ay)

!Regner ut fluksfunksjoner
!Flux i x-retning
F(:,:,1)=U(:,:,2)

F(:,:,2)=U(:,:,2)**2/U(:,:,1)+0.5*grav*U(:,:,1)**2   

F(:,:,3)=U(:,:,2)*U(:,:,3)/U(:,:,1)  

!Flux i y-retning
G(:,:,1)=U(:,:,3)  

G(:,:,2)=F(:,:,3)

G(:,:,3)=U(:,:,3)**2/U(:,:,1)+0.5*grav*U(:,:,1)**2   
 


!Regner ut Rusanov-flukser
!fluks i x-retning
Flf(1,:,:)=F(1,:,:)

Flf(2,:,:)=F(1,:,:)

Flf(npi,:,:)=F(npi,:,:)


Flf(3:npi-1,:,:)=0.5*(F(2:npi-2,:,:)+F(3:npi-1,:,:)-ax(3:npi-1,:)*(Un(3:npi-1,:,:)-Un(2:npi-2,:,:)))



!fluks i y-retning
Glf(:,1,:)=G(:,1,:)

Glf(:,2,:)=G(:,1,:)

Glf(:,npj,:)=G(:,npj,:)

Glf(:,3:npj-1,:)=0.5*(G(:,2:npj-2,:)+G(:,3:npj-1,:)-ay(:,3:npj-1)*(Un(:,3:npj-1,:)-Un(:,2:npj-2,:)))

CALL outblock_and_source(dx,dy)


END SUBROUTINE local_lf

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE make_diffterm(Un,a_llf_x,a_llf_y)
USE declarations
IMPLICIT NONE

INTEGER											 		:: i
DOUBLE PRECISION, DIMENSION(npi,npj)					:: eigen_x1, eigen_x2, eigen_y1, eigen_y2
DOUBLE PRECISION, DIMENSION(npi,npj), INTENT(out)		:: a_llf_x, a_llf_y
DOUBLE PRECISION, DIMENSION(npi,npj)					:: spectral_x, spectral_y
DOUBLE PRECISION, INTENT(in), DIMENSION(npi,npj,3)		:: Un
!-------------------------------------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Regner ut diffusiv konstant a(i+1/2) og a(j+1/2)
!-------------------------------------------------------------------------------------------------

a_llf_x=0.
a_llf_y=0.

eigen_x1=Un(:,:,2)/Un(:,:,1)-SQRT(grav*Un(:,:,1))
eigen_x2=Un(:,:,2)/Un(:,:,1)+SQRT(grav*Un(:,:,1))
eigen_y1=Un(:,:,3)/Un(:,:,1)-SQRT(grav*Un(:,:,1))
eigen_y2=Un(:,:,3)/Un(:,:,1)+SQRT(grav*Un(:,:,1))
spectral_x=max(abs(eigen_x1),abs(eigen_x2))
spectral_y=max(abs(eigen_y1),abs(eigen_y2))

DO i=3,npi-1
  a_llf_x(i,:)=max(spectral_x(i-1,:),spectral_x(i,:))
END DO

DO i=3,npj-1
  a_llf_y(:,i)=max(spectral_y(:,i-1),spectral_y(:,i))
  
END DO

END SUBROUTINE make_diffterm

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE bound()

USE declarations
IMPLICIT NONE
!---------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Setter randbetingesler (null-grad.)
!---------------------------------------------------------------------

!pne randbetingelser
U(1,:,:)=U(2,:,:)
U(npi,:,:)=U(npi-1,:,:)
U(2:npi-1,1,:)=U(2:npi-1,2,:)
U(2:npi-1,npj,:)=U(2:npi-1,npj-1,:)

END SUBROUTINE bound

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE explicit_solver(dx,dy,i)
USE declarations
IMPLICIT NONE

DOUBLE PRECISION, INTENT (in)     		:: dx,dy
INTEGER 						  		:: iter,j,ii
INTEGER, INTENT(out)  			  		:: i
DOUBLE PRECISION, DIMENSION(npi,npj)  	:: u_old, v_old, uabs_old
!---------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Eksplisitt masjering i tid 
!---------------------------------------------------------------------

iter=FLOOR(t_end/dt)

tid=0.0000

CALL printout(0)
WRITE(*,*)
WRITE(*,*)'Itererer...'
WRITE(*,*)

!Marsjerer fram i tid ved explicit euler
DO i=1,iter
  

masse_tot=0.

!regner sammen masse i dammen ved iterasjon i:
DO ii = 1,npi
  DO j= 1,npj
    IF (U(ii,j,1)>eps) THEN
      masse_tot=masse_tot+U(ii,j,1)*dx*dy*varme_o%rho_m
      END IF
  END DO
END DO 



    iter_pub=i
    
	tid=i*dt
    
   	u_old=U(:,:,2)/U(:,:,1) 
    
    v_old=U(:,:,3)/U(:,:,1) 
   
    uabs_old=SQRT(u_old**2+v_old**2)

    CALL RK_TVD1(dx,dy)
 

    CALL RK_back(uabs_old)

	CALL bound() 

    CALL printout(i)
    
	tid_vek(i+1)=tid
    
    !Tar vare p avdampet masse i vektor
 	mass_tot(i)=masse_tot
    !massefluks i nsket celle
    mass_rate_tot(i+1)=SUM(m_vap)*dx*dy
    
    h_last=U(:,:,1) !holder orden p hyden ved forrige tidsskritt

END DO


END SUBROUTINE explicit_solver

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE residuals1(res,dx,dy,Un,first,S)
USE declarations
IMPLICIT NONE

DOUBLE PRECISION, INTENT (in)     					 :: dx,dy
DOUBLE PRECISION									 :: As, An, Aw, Ae						
DOUBLE PRECISION, INTENT(out), DIMENSION(npi,npj,3)	 :: res
DOUBLE PRECISION, INTENT(in), DIMENSION(npi,npj,3)	 :: Un
DOUBLE  PRECISION, INTENT(inout), DIMENSION(npi,npj) :: S
INTEGER												 :: first
!---------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Regner ut residualene som brukes i RK
!---------------------------------------------------------------------
As=dx
An=As
Aw=dy
Ae=Aw

IF (determ1=='y') THEN


CALL local_lf(Un,dx,dy)

res(2:npi-1,2:npj-1,:)=(1/(dx*dy))*(Aw*Flf(2:npi-1,2:npj-1,:) &
    & -Ae*Flf(3:npi,2:npj-1,:)+As*Glf(2:npi-1,2:npj-1,:)-An*Glf(2:npi-1,3:npj,:))
    
res(1,:,:)=0.
res(npi,:,:)=0.
res(:,1,:)=0.
res(:,npj,:)=0.

ELSEIF (determ1=='n') THEN


res=0. ! skrur av spredningen

ELSE 
  
WRITE(*,*)'Svar enten med "y" eller "n"'

STOP
END IF

IF (determ2=='y') THEN

!IF (first==1) THEN
CALL sourceterm_evap(S,U,dx,dy)
!END IF

res(:,:,1)=res(:,:,1)+S

END IF

END SUBROUTINE residuals1

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE RK_TVD1(dx,dy)
USE declarations
IMPLICIT NONE

DOUBLE PRECISION, DIMENSION(npi,npj,3)				:: res1, U1, U2
DOUBLE PRECISION, INTENT (in)     					:: dx, dy
DOUBLE  PRECISION, DIMENSION(npi,npj) 				:: S
!---------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Eksplisitt RK-lser (Shu mfl)
!---------------------------------------------------------------------

!Setter initiell verdi for kilde:
S=0.

CALL residuals1(res1,dx,dy,U,1,S)


U1=U+dt*res1


CALL residuals1(res1,dx,dy,U1,0,S)


U2=0.75*U+0.25*U1+0.25*dt*res1


CALL residuals1(res1,dx,dy,U2,0,S)


U=(1./3)*U+(2./3)*U2+(2./3)*dt*res1


END SUBROUTINE RK_TVD1

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE RK_back(uabs_old) !implisitt
USE declarations
IMPLICIT NONE

DOUBLE PRECISION, DIMENSION(npi,npj):: uabs_old, deter_lam_x, deter_lam_y, deter_turb_x,deter_turb_y
REAL, PARAMETER						:: CT=0.0015*1.7, CL=3, k_visk=4.3e-07 !m2/s inntil videre konstant
INTEGER								:: i,j
!------------------------------------------------------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Semi-implisitt metode for frisksjonsledd
!------------------------------------------------------------------------------------------------------------------

deter_lam_x=abs(-CL*k_visk*U(:,:,2)/U(:,:,1)**2)
deter_lam_y=abs(-CL*k_visk*U(:,:,3)/U(:,:,1)**2)
deter_turb_x=abs(-CT*U(:,:,2)*uabs_old/U(:,:,1))
deter_turb_y=abs(-CT*U(:,:,3)*uabs_old/U(:,:,1))

DO i=1,npi
  DO j=1,npj
    
!x-retning
IF (deter_turb_x(i,j)>deter_lam_x(i,j)) THEN
  
	U(i,j,2)=U(i,j,2)/(1+CT*dt*uabs_old(i,j)/U(i,j,1))
ELSE 
  
 	U(i,j,2)=U(i,j,2)/(1+CL*k_visk*dt/(U(i,j,1)**2))
END IF

!y-retning
IF (deter_turb_y(i,j)>deter_lam_y(i,j)) THEN
	U(i,j,3)=U(i,j,3)/(1+CT*dt*uabs_old(i,j)/U(i,j,1))
ELSE 
 	U(i,j,3)=U(i,j,3)/(1+CL*k_visk*dt/(U(i,j,1)**2))
END IF


  END DO
END DO	 


END SUBROUTINE RK_back

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

SUBROUTINE sourceterm_evap(S,Un,dx,dy)
USE declarations
IMPLICIT NONE

DOUBLE  PRECISION, INTENT(out), DIMENSION(npi,npj)  	:: S  !Kildeledd (regresjonshastighet(1,2), varmefluks(3), massefluks(4))
DOUBLE  PRECISION, INTENT(in), DIMENSION(npi,npj,3)		:: Un
INTEGER													:: iter,i,j
DOUBLE PRECISION										:: dx,dy,init_mass,L,Re_L,Nu_L,h_a 
REAL													:: bak
REAL, PARAMETER											:: L1=1., vind=3. 
!---------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Regner ut kildeleddet(reg.hast) som flger av varmeoverfring
!---------------------------------------------------------------------

iter=CEILING(tid/dt)
init_mass=0.

!initierer varmeovergangstall
L=10.
bak=dia_t
Re_L=(vind*L)/(varme_o%ny_a)
Nu_L=0.037*(Re_L**0.8)*(varme_o%Pr_a)**(1./3)
h_a=(varme_o%k_a)*Nu_L/L
	


!initeierer q_u hver gang
q_u=0.

DO i=1,npi
  		DO j=1,npj
		
	IF (i>1 .and. i<npi) THEN
    
    	IF (Un(i+1,npj/2,1)>=2*eps .and. Un(i-1,npj/2,1)<2*eps .and. i>cell_rel)THEN
      		bak=x(i)! tracer bakside av dammen
    	END IF 
        
		IF (Un(i+1,npj/2,1)<=2*eps .and. Un(i-1,npj/2,1)>2*eps)THEN
        
          IF ((x(i)-bak)>L1) THEN
            
          		L=x(i)-bak
               
          END IF
          
		Re_L=(vind*L)/(varme_o%ny_a)
        Nu_L=0.037*(Re_L**0.8)*(varme_o%Pr_a)**(1./3)
		h_a=(varme_o%k_a)*Nu_L/L		
		
        END IF  
	END IF
    
		IF (U(i,j,1)>2*eps) THEN !Hvis vske til stede i celle ==>varmefluks/avdampning
          
		
 		q_u(i,j)=(SQRT(varme_o%gamma*varme_o%rho_g*varme_o%Cp_g/pi))*(varme_o%T_jord-T_dam)*tid_evap(i,j)**(-0.5) 
        
		!Legger til konvektiv varme fra luft
        q_u(i,j)=q_u(i,j)+h_a*(varme_o%T_a-T_dam)
         
		tid_evap(i,j)=tid_evap(i,j)+dt
        
		END IF
        IF(U(i,j,1)<=2*eps)THEN
          U(i,j,1)=eps
          q_u(i,j)=0.
          s_vap(i,j)=0.
          m_vap(i,j)=0.
        END IF
		END DO        
END DO

!Lagrer konvektiv varmeovergangskoeff i vektor
h_a_vek(iter_pub)=h_a
!Lagrer varme overfrt til dammen i vektor
q_tot(iter_pub+1)=SUM(q_u)*dx*dy ![Watt]


!Massefluks avdampet
m_vap=q_u/varme_o%h_fgm

!Kilde i kontinuitetsligningen
s_vap=q_u/(varme_o%rho_m*varme_o%h_fgm)

! Veleger fysisk reg. hast. eller "resten"
    S=-min(0.9*abs(Un(:,:,1)-eps)/dt,s_vap)

s_vap_e=S
mass_rate_tot(iter_pub)=SUM(S)*dx*dy*varme_o%rho_m
mass_tot_avd(iter_pub+1)=mass_tot_avd(iter_pub)+dt*mass_rate_tot(iter_pub)

END SUBROUTINE sourceterm_evap

END MODULE sub

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

PROGRAM hoved

USE declarations
USE sub
IMPLICIT NONE

DOUBLE PRECISION :: dx,dy
INTEGER 		 :: i, iter
!---------------------------------------------------------------------
!Skrevet av: Anders Welde Vikan
!Hensikt: Hovedprogram (kjrer subrutinene)
!---------------------------------------------------------------------

CALL read_prop()

CALL grid(dx,dy)

CALL init(dx,dy)

CALL explicit_solver(dx,dy,i)


OPEN(19,file='tid.dat',status='unknown')
OPEN(20,file='mass_tot.dat',status='unknown')
OPEN(21,file='mass_rate_tot.dat',status='unknown')
OPEN(22,file='h_a.dat',status='unknown')
OPEN(23,file='q_tot.dat',status='unknown')
OPEN(24,file='mass_rate_tot2.dat',status='unknown')

iter=CEILING(t_end/dt)
WRITE(*,*) iter

DO i=1,iter+1

  WRITE(19,100) tid_vek(i)
  WRITE(20,100) mass_tot(i)
  WRITE(21,100) mass_rate_tot(i)
  WRITE(22,100) h_a_vek(i)
  WRITE(23,100) q_tot(i)
  WRITE(24,100) mass_tot_avd(i)
END DO

WRITE(*,*)size(tid_vek) 
WRITE(*,*)size(mass_tot)

CLOSE(19)
CLOSE(20)
CLOSE(21)
CLOSE(22)
CLOSE(23)
CLOSE(24)
100  FORMAT(1E15.3)
WRITE(*,*)
WRITE(*,*)'masse_tot.dat og tid.dat skrevet ut.'

STOP
END PROGRAM hoved

!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC